@@ -1,5 +1,16 @@
 #if (!isGeneric("#")) { setGeneric("#", function(x, ...) standardGeneric("#")) }
 
+if (!isGeneric("panel")) {setGeneric("panel", function(x, ...) standardGeneric("panel"))}
+#if (!isGeneric("colSums")) {setGeneric("colSums", function(x, ...) standardGeneric("colSums"))}	
+#if (!isGeneric("rowSums")) {setGeneric("rowSums", function(x, ...) standardGeneric("rowSums"))}	
+#if (!isGeneric("colMeans")) {setGeneric("colMeans", function(x, ...) standardGeneric("colMeans"))}	
+#if (!isGeneric("rowMeans")) {setGeneric("rowMeans", function(x, ...) standardGeneric("rowMeans"))}	
+
+if (!isGeneric("logic")) {setGeneric("logic", function(x, ...) standardGeneric("logic"))}
+if (!isGeneric("compare")) {setGeneric("compare", function(x, y, ...) standardGeneric("compare"))}
+
+if (!isGeneric("meta")) {setGeneric("meta", function(x, ...) standardGeneric("meta"))}
+
 if (!isGeneric("rangeFill")) {setGeneric("rangeFill", function(x, ...) standardGeneric("rangeFill"))}
 if (!isGeneric("roll")) {setGeneric("roll", function(x, ...) standardGeneric("roll"))}
 if (!isGeneric("elongate")) {setGeneric("elongate", function(x, ...) standardGeneric("elongate"))}
@@ -224,6 +235,7 @@ if (!isGeneric("boundaries")) {	setGeneric("boundaries", function(x, ...) standa
 if (!isGeneric("boxplot")) { setGeneric("boxplot", function(x, ...) standardGeneric("boxplot"))}
 if (!isGeneric("buffer")) {setGeneric("buffer", function(x, ...) standardGeneric("buffer"))}
 if (!isGeneric("clamp")) { setGeneric("clamp", function(x, ...) standardGeneric("clamp")) }
+if (!isGeneric("clamp_ts")) { setGeneric("clamp_ts", function(x, ...) standardGeneric("clamp_ts")) }
 if (!isGeneric("click")) {setGeneric("click", function(x, ...)standardGeneric("click"))}
 if (!isGeneric("contour")) { setGeneric("contour", function(x,...) standardGeneric("contour"))}
 if (!isGeneric("cover")) {setGeneric("cover", function(x, y, ...) standardGeneric("cover"))}
