@@ -9,7 +9,7 @@
 
 if (!isGeneric("layerCor")) {setGeneric("layerCor", function(x, ...) standardGeneric("layerCor"))}
 
-setMethod("layerCor", signature(x="SpatRaster"), 
+setMethod("layerCor", signature(x="SpatRaster"),
 	function(x, fun, w, asSample=TRUE, na.rm=FALSE, maxcell=Inf, ...) {
 
 		stopifnot(is.logical(asSample) & !is.na(asSample))
@@ -33,23 +33,21 @@ setMethod("layerCor", signature(x="SpatRaster"),
 			}
 			stopifnot( nlyr(w) == 1 )
 
-			if (na.rm) {
-			# a cell is set to NA if it is NA in any layer. That is not ideal, but easier and quicker
-				nas <- sum(x) * w
-				x <- mask(x, nas)
-				w <- mask(w, nas)
-			}
-
 			sumw <- unlist(global(w, fun="sum", na.rm=na.rm) )
 			means <- unlist(global(x * w, fun="sum", na.rm=na.rm)) / sumw
 			sumw <- sumw - asSample
-
 			x <- (x - means) * sqrt(w)
 
-
 			for(i in 1:nl) {
 				for(j in i:nl) {
-					r <- x[[i]] * x[[j]]
+					if (na.rm) {
+						m <- anyNA(x[[c(i,j)]])
+						a <- mask(x[[i]], m, maskvalue=TRUE)
+						b <- mask(x[[j]], m, maskvalue=TRUE)
+						r <- a * b
+					} else {
+						r <- x[[i]] * x[[j]]
+					}
 					v <- unlist(global(r, fun="sum", na.rm=na.rm)) / sumw
 					mat[j,i] <- mat[i,j] <- v
 				}
@@ -66,10 +64,14 @@ setMethod("layerCor", signature(x="SpatRaster"),
 
 			for(i in 1:nl) {
 				for(j in i:nl) {
-					r <- x[[i]] * x[[j]]
 					if (na.rm) {
+						m <- anyNA(x[[c(i,j)]])
+						a <- mask(x[[i]], m, maskvalue=TRUE)
+						b <- mask(x[[j]], m, maskvalue=TRUE)
+						r <- x[[i]] * x[[j]]
 						v <- unlist(global(r, fun="sum", na.rm=na.rm)) / (n - unlist(global(r, fun="isNA")) - asSample)
 					} else {
+						r <- x[[i]] * x[[j]]
 						v <- unlist(global(r, fun="sum", na.rm=na.rm)) / (n - asSample)
 					}
 					mat[j,i] <- mat[i,j] <- v
@@ -82,22 +84,34 @@ setMethod("layerCor", signature(x="SpatRaster"),
 			return(covar)
 
 		} else if (fun == "pearson") {
-
-			means <- unlist(global(x, fun="mean", na.rm=na.rm) )
-			sds <- unlist(global(x, fun="sd", na.rm=na.rm) )
-			x <- (x - means)
-
-			for(i in 1:nl) {
-				for(j in i:nl) {
-					r <- x[[i]] * x[[j]]
-					v <- unlist(global(r, fun="sum", na.rm=na.rm))
-					if (na.rm) {
+			if (na.rm) {
+				for(i in 1:nl) {
+					for(j in i:nl) {
+						m <- anyNA(x[[c(i,j)]])
+						a <- mask(x[[i]], m, maskvalue=TRUE)
+						b <- mask(x[[j]], m, maskvalue=TRUE)
+						xx <- c(a, b)
+						means <- unlist(global(xx, fun="mean", na.rm=na.rm) )
+						sds <- unlist(global(xx, fun="sd", na.rm=na.rm) )
+						r <- prod(xx - means)
 						nas <- unlist(global(is.na(r), fun="sum"))
+						v <- unlist(global(r, fun="sum", na.rm=na.rm))
 						v <- v / ((n - nas - asSample) * sds[i] * sds[j])
-					} else {
+						mat[j,i] <- mat[i,j] <- v
+					}
+				}
+			} else {
+				means <- unlist(global(x, fun="mean", na.rm=na.rm) )
+				sds <- unlist(global(x, fun="sd", na.rm=na.rm) )
+				x <- (x - means)
+				
+				for(i in 1:nl) {
+					for(j in i:nl) {
+						r <- x[[i]] * x[[j]]
+						v <- unlist(global(r, fun="sum", na.rm=na.rm))
 						v <- v / ((n - asSample) * sds[i] * sds[j])
+						mat[j,i] <- mat[i,j] <- v
 					}
-					mat[j,i] <- mat[i,j] <- v
 				}
 			}
 			names(means) <- names(x)
@@ -107,14 +121,13 @@ setMethod("layerCor", signature(x="SpatRaster"),
 
 		} else {
 
-			v <- spatSample(x, size=maxcell, "regular", na.rm=na.rm)
+			v <- spatSample(x, size=maxcell, "regular", na.rm=na.rm, warn=FALSE)
 			for(i in 1:nl) {
 				for(j in i:nl) {
-					mat[j,i] <- mat[i,j] <- FUN(v[,i], v[,j])
+					mat[j,i] <- mat[i,j] <- FUN(v[,i], v[,j], ...)
 				}
 			}
 			mat
-
 		}
 	}
 )
