@@ -54,11 +54,7 @@
 
 
 
-.write_cdf <- function(x, filename, overwrite=FALSE, zname="time", atts="", gridmap="", prec="float", compression=NA, missval, ...) {
-
-	dots <- list(...)
-	force_v4 <- isTRUE(dots$force_v4)
-	verbose  <- isTRUE(dots$verbose)
+.write_cdf <- function(x, filename, overwrite=FALSE, zname="time", atts="", gridmap="", prec="float", compression=NA, missval, force_v4=TRUE, verbose=FALSE, ...) {
 
 	n <- length(x)
 	y <- x[1]
@@ -114,6 +110,10 @@
 					zunit <- "days since 1970-1-1"
 					zv <- zv / (24 * 3600)
 					cal <- "standard"
+				} else if (tstep == "years") {
+					zunit <- "years since 1970"
+					zv <- time(y) - 1970
+					cal <- "standard"
 				} else {
 					zunit <- "unknown"
 				}
@@ -129,7 +129,6 @@
 		}
 	}
 
-
 	ncvars[[n+1]] <- ncdf4::ncvar_def("crs", "", list(), NULL, prec="integer")
 	
 	ncobj <- ncdf4::nc_create(filename, ncvars, force_v4=force_v4, verbose=verbose)
