@@ -14,13 +14,16 @@
 #include "string_utils.h"
 
 
-bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std::vector<size_t> xyz) {
+bool SpatRaster::constructFromFileMulti(std::string fname, std::vector<int> sub, std::vector<std::string> subname, std::vector<std::string> drivers, std::vector<std::string> options, std::vector<size_t> xyz) {
+
+Rcpp::Rcout << "in" << std::endl;
 
 	if (xyz.size() != 3) {
 		setError("you must supply three dimension indices");
         return false;
 	}
-	
+
+
     auto poDataset = std::unique_ptr<GDALDataset>(
         GDALDataset::Open(fname.c_str(), GDAL_OF_MULTIDIM_RASTER ));
     if( !poDataset ) {
@@ -34,23 +37,56 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 		return false;
     }
 
-	bool warngroup = false;
+
 	std::vector<std::string> gnames;
-	if (sub == "") {
-		char** papszOptions = NULL;
-		gnames = poRootGroup->GetMDArrayNames(papszOptions);
-		CSLDestroy(papszOptions);
-		sub = gnames[0];
-		if (gnames.size() > 1) warngroup = true;
+	std::string subdsname = "";
+	char** papszOptions = NULL;
+	gnames = poRootGroup->GetMDArrayNames(papszOptions);
+	CSLDestroy(papszOptions);
+
+	if (gnames.size() == 0) {
+		setError("no subdatsets detected");
+		return false;
+	}
+	Rcpp::Rcout << "available: ";
+	for (size_t i=0; i<gnames.size(); i++) {
+		Rcpp::Rcout << gnames[i] << " ";
+	}
+	Rcpp::Rcout << std::endl;
+
+
+	if (subname.size() > 0) {
+		subdsname = subname[0];
+		if (std::find(gnames.begin(), gnames.end(), subdsname) == gnames.end()) {
+			setError("subdatset name not found");
+			return false;
+		}
+	} else if (sub[0] >= 0) {
+		if (sub[0] >= (int)gnames.size()) {
+			setError("subdatset is out or range");
+			return false;
+		} else {
+			subdsname = gnames[sub[0]];
+		}
+	} else {
+		subdsname = gnames[0];
+		if (gnames.size() > 1)  {
+			std::string gn = "";
+			for (size_t i=1; i<gnames.size(); i++) {
+				gn += gnames[i] + ", ";
+			}
+			addWarning("using: " + subdsname + ". Other groups are: \n" + gn);
+		}
 	}
 
-    auto poVar = poRootGroup->OpenMDArray(sub.c_str());
+	Rcpp::Rcout << "subdsname: " << subdsname << std::endl;
+
+    auto poVar = poRootGroup->OpenMDArray(subdsname.c_str());
     if( !poVar )   {
-		setError("cannot find: " + sub);
+		setError("cannot find: " + subdsname);
 		return false;
     }
 
-	SpatRasterSource s;
 
 	std::string wkt = "";
 	std::shared_ptr<OGRSpatialReference> srs = poVar->GetSpatialRef();
@@ -60,15 +96,17 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 		OGRErr err = srs->exportToWkt(&cp, options);
 		if (err == OGRERR_NONE) {
 			wkt = std::string(cp);
-		} 
+		}
 		CPLFree(cp);
 	}
+
+	SpatRasterSource s;
+
 	std::string msg;
 	if (!s.srs.set({wkt}, msg)) {
 		addWarning(msg);
 	}
 
-
 	std::vector<size_t> dimcount;
 	std::vector<std::string> dimnames;
 	std::vector<double> dim_start, dim_end;
@@ -84,7 +122,7 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 		const auto indvar = poDim->GetIndexingVariable();
 		indvar->Read(
 			std::vector<GUInt64>{0}.data(),
-            count.data(), nullptr, nullptr, 
+            count.data(), nullptr, nullptr,
             GDALExtendedDataType::Create(GDT_Float64),
             &vals[0]);
 
@@ -94,20 +132,10 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 		Rcpp::Rcout << vals[0] << " - " << vals[vals.size()-1] << std::endl;
 
     }
-    GDALExtendedDataTypeRelease(hDT); 	
+    GDALExtendedDataTypeRelease(hDT);
 
 	s.m_ndims = dimcount.size();
-
-	if (warngroup) {
-		std::string gn = "";
-		for (size_t i=0; i<gnames.size(); i++) {
-			if (!is_in_vector(gnames[i], dimnames) &&  (gnames[i] != sub)) { 
-				gn += gnames[i] + ", ";
-			}
-		}
-		addWarning("using: " + sub + ". Other groups are: \n" + gn);
-	}
-	s.source_name = sub;
+	s.source_name = subdsname;
 	s.source_name_long = poVar->GetAttribute("long_name")->ReadAsString();
 
 	s.m_hasNA = false;
@@ -125,7 +153,7 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 		e.ymin = dim_end[xyz[0]] - 0.5 * res;
 	} else {
 		setError("the second dimension is not valid");
-		return false;		
+		return false;
 	}
 	if (xyz[1] < s.m_ndims) {
 		s.ncol = dimcount[xyz[1]];
@@ -135,7 +163,7 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 		e.xmax = dim_end[xyz[1]] + 0.5 * res;
 	} else {
 		setError("the first dimension is not valid");
-		return false;		
+		return false;
 	}
 	if (s.m_ndims > 2) {
 		if (xyz[2] < s.m_ndims) {
@@ -143,11 +171,11 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 			s.m_dimnames.push_back(dimnames[xyz[2]]);
 		} else {
 			setError("the third dimension is not valid");
-			return false;		
+			return false;
 		}
 	}
 	s.m_dims = xyz;
-	s.extent = e;	
+	s.extent = e;
 	if (s.m_ndims > 3) {
 		for (size_t i=0; i<s.m_ndims; i++) {
 			bool found = false;
@@ -156,12 +184,13 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 			}
 			if (!found) {
 				s.m_dims.push_back(i);
-				s.m_dimnames.push_back(dimnames[i]);				
+				s.m_dimnames.push_back(dimnames[i]);
 			}
 		}
 	}
-	
+
 	s.nlyrfile = s.nlyr;
+	s.resize(s.nlyr);
 	s.layers.resize(s.nlyr);
     std::iota(s.layers.begin(), s.layers.end(), 0);
 	s.flipped = false;
@@ -173,7 +202,9 @@ bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std:
 	s.multidim = true;
 
 // layer names
-// time 
+	//std::vector<std::string> nms(s.nlyr, "");
+	//s.names = nms;
+// time
 // extent
 	s.m_counts = dimcount;
 	setSource(s);
@@ -198,7 +229,7 @@ bool SpatRaster::readStartMulti(unsigned src) {
 		setError("not a good root group");
 		return false;
     }
-    
+
 	GDALMDArrayH hVar = GDALGroupOpenMDArray(hGroup, source[src].source_name.c_str(), NULL);
     GDALGroupRelease(hGroup);
     if (!hVar) {
@@ -219,30 +250,31 @@ bool SpatRaster::readStopMulti(unsigned src) {
 
 
 bool SpatRaster::readValuesMulti(std::vector<double> &out, size_t src, size_t row, size_t nrows, size_t col, size_t ncols) {
-	
+
+	Rcpp::Rcout << "reading" << std::endl;
 
 	std::vector<GUInt64> offset(source[src].m_ndims, 0);
 	std::vector<size_t> dims = source[src].m_dims;
 
-	offset[source[src].m_dims[0]] = row;
-	offset[source[src].m_dims[1]] = col;
+	offset[source[src].m_dims[0]] = col;
+	offset[source[src].m_dims[1]] = row;
 	offset[source[src].m_dims[2]] = 0;
 
 //	std::vector<size_t> count = source[src].m_counts;
 	std::vector<size_t> count(source[src].m_ndims, 1);
-	count[source[src].m_dims[0]] = nrows;
-	count[source[src].m_dims[1]] = ncols;
+	count[source[src].m_dims[0]] = ncols;
+	count[source[src].m_dims[1]] = nrows;
 	count[source[src].m_dims[2]] = nlyr();
 
 	size_t n=1;
 	for (size_t i=0; i<count.size(); i++) {
 		n *= count[i];
 	}
-	
+
 	//count = {3600, 1, 1, 7200, 1};
 
 	GDALExtendedDataTypeH hDT = GDALExtendedDataTypeCreate(GDT_Float64);
-	
+
 	std::vector<double> temp;
 	temp.resize(n);
 	GDALMDArrayRead(source[src].gdalmdarray,
@@ -252,32 +284,39 @@ bool SpatRaster::readValuesMulti(std::vector<double> &out, size_t src, size_t ro
 						NULL, // stride: default to row-major convention
 						hDT,
 						&temp[0],
-						NULL, // array start. Omitted 
-						0 // array size in bytes. Omitted 
+						NULL, // array start. Omitted
+						0 // array size in bytes. Omitted
 						);
-    GDALExtendedDataTypeRelease(hDT); 	
-	
-	size_t nc = ncell();
-	size_t nl = nlyr();
-	out.resize(0);
-	out.reserve(n);
-	for (size_t i=0; i<nl; i++) {
-		for (size_t j=0; j<nc; j++) {
-			out.push_back( temp[nl*j + i] );
-		}
-	}
+    GDALExtendedDataTypeRelease(hDT);
 
+
+//tbd: row order should be reversed
+
+//	size_t nc = nrows * ncols;
+//	size_t nl = nlyr();
+//	out.resize(0);
+//	out.reserve(n);
+//	for (size_t i=0; i<nl; i++) {
+//		for (size_t j=0; j<nc; j++) {
+//			out.push_back( temp[nl*j + i] );
+//		}
+//	}
+
+
+    out = std::move(temp);
 	if (source[src].m_hasNA) {
 		std::replace (out.begin(), out.end(), source[src].m_missing_value, (double)NAN);
 	}
+
 	return true;
 }
 
-  
-#else  
+
+#else
 
 */
-bool SpatRaster::constructFromFileMulti(std::string fname, std::string sub, std::vector<size_t> xyz) {
+
+bool SpatRaster::constructFromFileMulti(std::string fname, std::vector<int> sub, std::vector<std::string> subname, std::vector<std::string> drivers, std::vector<std::string> options, std::vector<size_t> xyz) {
 	setError("multidim is not supported by GDAL < 3.1");
 	return false;
 }
@@ -297,7 +336,6 @@ bool SpatRaster::readValuesMulti(std::vector<double> &out, size_t src, size_t ro
 	return false;
 }
 
-/*
-#endif
 
-*/
+//#endif
+
