@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2022  Robert J. Hijmans
+// Copyright (c) 2018-2023  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -21,6 +21,10 @@
 #include "recycle.h"
 #include "string_utils.h"
 
+void callbck(void *item, void *userdata) { // callback function for tree selection
+	std::vector<size_t> *ret = (std::vector<size_t> *) userdata;
+	ret->push_back(*((size_t *) item));
+}
 
 SpatVector SpatVector::allerretour() {
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
@@ -316,7 +320,7 @@ SpatVector SpatVector::crop(SpatExtent e) {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt, id);
 		out = coll.get(0);
 		out.df = df.subset_rows(out.df.iv[0]);
@@ -351,7 +355,7 @@ SpatVector SpatVector::make_nodes() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -385,7 +389,7 @@ SpatVector SpatVector::boundary() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -447,7 +451,7 @@ SpatVector SpatVector::line_merge() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -484,7 +488,7 @@ SpatVector SpatVector::simplify(double tolerance, bool preserveTopology) {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.df = df;
@@ -495,12 +499,11 @@ SpatVector SpatVector::simplify(double tolerance, bool preserveTopology) {
 }
 
 
-
-SpatVector SpatVector::shared_paths() {
+SpatVector SpatVector::shared_paths(bool index) {
 
 	if (type() == "polygons") {
 		SpatVector x = as_lines();
-		return x.shared_paths();
+		return x.shared_paths(index);
 	}
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
@@ -509,23 +512,59 @@ SpatVector SpatVector::shared_paths() {
 	size_t s = size();
 	std::vector<long> id1, id2;
 	std::vector<GeomPtr> p;
-	for (size_t i=0; i<(s-1); i++) {
-		for (size_t j=(i+1); j<s; j++) {
-			GEOSGeometry* r = GEOSSharedPaths_r(hGEOSCtxt, x[i].get(), x[j].get());
-			if (r != NULL) {
-				if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
-					p.push_back(geos_ptr(r, hGEOSCtxt));
-					id1.push_back(i+1);
-					id2.push_back(j+1);
-				} else {
-					GEOSGeom_destroy_r(hGEOSCtxt, r);
+
+	if (!index) {
+		// calculate shared paths
+		for (size_t i=0; i<(s-1); i++) {
+			for (size_t j=(i+1); j<s; j++) {
+				GEOSGeometry* r = GEOSSharedPaths_r(hGEOSCtxt, x[i].get(), x[j].get());
+				if (r != NULL) {
+					if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
+						p.push_back(geos_ptr(r, hGEOSCtxt));
+						id1.push_back(i+1);
+						id2.push_back(j+1);
+					} else {
+						GEOSGeom_destroy_r(hGEOSCtxt, r);
+					}
+				}
+			}
+		}
+	} else {
+		// use spatial index
+		std::vector<size_t> items(x.size());
+		TreePtr tree1 = geos_ptr(GEOSSTRtree_create_r(hGEOSCtxt, 10), hGEOSCtxt);
+		for (size_t i = 0; i < s; i++) {
+			items[i] = i;
+			if (! GEOSisEmpty_r(hGEOSCtxt, x[i].get()))
+				GEOSSTRtree_insert_r(hGEOSCtxt, tree1.get(), x[i].get(), &(items[i]));
+		}
+		for (size_t i = 0; i < s; i++) {
+			// pre-select x's using tree:
+			std::vector<size_t> tree_sel, sel;
+			if (!GEOSisEmpty_r(hGEOSCtxt, x[i].get())) {
+				GEOSSTRtree_query_r(hGEOSCtxt, tree1.get(), x[i].get(), callbck, &tree_sel);
+			}
+			if (! tree_sel.empty()) {
+				for (size_t j = 0; j < tree_sel.size(); j++) {
+					if (tree_sel[j] > i) {
+						GEOSGeometry* r = GEOSSharedPaths_r(hGEOSCtxt, x[i].get(), x[tree_sel[j]].get());
+						if (r != NULL) {
+							if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
+								p.push_back(geos_ptr(r, hGEOSCtxt));
+								id1.push_back(i+1);
+								id2.push_back(tree_sel[j]+1);
+							} else {
+								GEOSGeom_destroy_r(hGEOSCtxt, r);
+							}
+						}
+					}
 				}
 			}
 		}
 	}
 
 	SpatVector out;
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt, std::vector<long>(), false, false);
 		out = coll.get(0);
 		out = out.line_merge();
@@ -538,14 +577,14 @@ SpatVector SpatVector::shared_paths() {
 }
 
 
-SpatVector SpatVector::shared_paths(SpatVector x) {
+SpatVector SpatVector::shared_paths(SpatVector x, bool index) {
 
 	if (x.type() == "polygons") {
 		x = x.as_lines();
 	}
 	if (type() == "polygons") {
 		SpatVector v = as_lines();
-		return v.shared_paths(x);
+		return v.shared_paths(x, index);
 	}
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
@@ -556,23 +595,56 @@ SpatVector SpatVector::shared_paths(SpatVector x) {
 	size_t sb = b.size();
 	std::vector<long> id1, id2;
 	std::vector<GeomPtr> p;
-	for (size_t i=0; i<sa; i++) {
-		for (size_t j=0; j<sb; j++) {
-			GEOSGeometry* r = GEOSSharedPaths_r(hGEOSCtxt, a[i].get(), b[j].get());
-			if (r != NULL) {
-				if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
-					p.push_back(geos_ptr(r, hGEOSCtxt));
-					id1.push_back(i+1);
-					id2.push_back(j+1);
-				} else {
-					GEOSGeom_destroy_r(hGEOSCtxt, r);
+
+	if (!index) {
+		for (size_t i=0; i<sa; i++) {
+			for (size_t j=0; j<sb; j++) {
+				GEOSGeometry* r = GEOSSharedPaths_r(hGEOSCtxt, a[i].get(), b[j].get());
+				if (r != NULL) {
+					if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
+						p.push_back(geos_ptr(r, hGEOSCtxt));
+						id1.push_back(i+1);
+						id2.push_back(j+1);
+					} else {
+						GEOSGeom_destroy_r(hGEOSCtxt, r);
+					}
+				}
+			}
+		}
+	} else {
+		// use spatial index
+		std::vector<size_t> items(x.size());
+		TreePtr tree1 = geos_ptr(GEOSSTRtree_create_r(hGEOSCtxt, 10), hGEOSCtxt);
+		for (size_t i = 0; i < sb; i++) {
+			items[i] = i;
+			if (! GEOSisEmpty_r(hGEOSCtxt, b[i].get()))
+				GEOSSTRtree_insert_r(hGEOSCtxt, tree1.get(), b[i].get(), &(items[i]));
+		}
+		for (size_t i = 0; i < sa; i++) {
+			// pre-select x's using tree:
+			std::vector<size_t> tree_sel, sel;
+			if (!GEOSisEmpty_r(hGEOSCtxt, a[i].get())) {
+				GEOSSTRtree_query_r(hGEOSCtxt, tree1.get(), a[i].get(), callbck, &tree_sel);
+			}
+			if (! tree_sel.empty()) {
+				for (size_t j = 0; j < tree_sel.size(); j++) {
+					GEOSGeometry* r = GEOSSharedPaths_r(hGEOSCtxt, a[i].get(), b[tree_sel[j]].get());
+					if (r != NULL) {
+						if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
+							p.push_back(geos_ptr(r, hGEOSCtxt));
+							id1.push_back(i+1);
+							id2.push_back(tree_sel[j]+1);
+						} else {
+							GEOSGeom_destroy_r(hGEOSCtxt, r);
+						}
+					}
 				}
 			}
 		}
 	}
 
 	SpatVector out;
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt, std::vector<long>(), false, false);
 		out = coll.get(0);
 		out = out.line_merge();
@@ -626,7 +698,7 @@ SpatVector SpatVector::polygonize() {
 			GEOSGeom_destroy_r(hGEOSCtxt, r);
 		}
 	}
-	if (p.size() > 0) {
+	if (!p.empty()) {
 		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
 		out = coll.get(0);
 		out.srs = srs;
@@ -766,7 +838,7 @@ SpatVector SpatVector::crop(SpatVector v) {
 
 //	SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
 
-	if (result.size() > 0) {
+	if (!result.empty()) {
 //		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
 		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids);
 		out = coll.get(0);
@@ -783,7 +855,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 
 	SpatVector out;
 
-	if (by != "") {
+	if (!by.empty()) {
 		SpatVector tmp = aggregate(by, false);
 		if (tmp.hasError()) {
 			return tmp;
@@ -794,7 +866,7 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 			if (x.hasError()) {
 				return x;
 			}
-			if ((x.geoms.size() > 0) && (x.geoms[0].gtype == polygons)) {
+			if (!x.geoms.empty() && (x.geoms[0].gtype == polygons)) {
 				out.addGeom(x.geoms[0]);
 			} else {
 				SpatGeom g(polygons);
@@ -863,7 +935,9 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 	SpatVector a = aggregate(false);
 	std::vector<GeomPtr> g = geos_geoms(&a, hGEOSCtxt);
 	GEOSGeometry* v;
-	if (bnd.size() > 0) {
+	if (bnd.empty()) {
+		v = GEOSVoronoiDiagram_r(hGEOSCtxt, g[0].get(), NULL, tolerance, onlyEdges);
+	} else {
 		if (bnd.type() != "polygons") {
 			out.setError("boundary must be polygon");
 			geos_finish(hGEOSCtxt);
@@ -871,8 +945,6 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 		}
 		std::vector<GeomPtr> ge = geos_geoms(&bnd, hGEOSCtxt);
 		v = GEOSVoronoiDiagram_r(hGEOSCtxt, g[0].get(), ge[0].get(), tolerance, onlyEdges);
-	} else {
-		v = GEOSVoronoiDiagram_r(hGEOSCtxt, g[0].get(), NULL, tolerance, onlyEdges);
 	}
 	if (v == NULL) {
 		out.setError("GEOS exception");
@@ -887,7 +959,7 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 	out.srs = srs;
 	if (!out.hasError()) {
 		out = out.disaggregate(false);
-		if (bnd.size() > 0) {
+		if (!bnd.empty()) {
 			SpatDataFrame empty;
 			bnd.df = empty;
 			out = out.intersect(bnd, true);
@@ -970,7 +1042,7 @@ SpatVector lonlat_buf(SpatVector x, double dist, unsigned quadsegs, bool ispol,
 		std::vector<double> dd = destpoint_lonlat(0, halfy, 0, dist);
 		dist = dd[1] - halfy;
 		if (ishole) dist = -dist;
-		x = x.buffer({dist}, quadsegs);
+		x = x.buffer({dist}, quadsegs, "", "", NAN, false);	
 		x.srs = insrs;
 		return x;
 	}
@@ -1022,33 +1094,33 @@ SpatVector lonlat_buf(SpatVector x, double dist, unsigned quadsegs, bool ispol,
 }
 
 
-
-SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
+//, std::vector<std::string> pars
+SpatVector SpatVector::buffer(std::vector<double> d, unsigned quadsegs, std::string capstyle, std::string joinstyle, double mitrelimit, bool singlesided) {
 
 	quadsegs = std::min(quadsegs, (unsigned) 180);
 	SpatVector out;
 	out.srs = srs;
 	if (srs.is_empty()) {
-		out.setError("crs not defined");
-		return(out);
-	}
+		out.addWarning("unknown CRS. Results may be wrong");
+	} 
 	bool islonlat = is_lonlat();
-	if (dist.size() == 1 && dist[0] == 0) {
+	
+	if (d.size() == 1 && d[0] == 0) {
 		islonlat = false; //faster
 	}
 	std::string vt = type();
 	if (vt == "points" || vt == "lines") {
-		for (size_t i=0; i<dist.size(); i++) {
-			if (dist[i] <= 0) {
-				dist[i] = -dist[i];
+		for (size_t i=0; i<d.size(); i++) {
+			if (d[i] <= 0) {
+				d[i] = -d[i];
 			}
 		}
 	}
-	recycle(dist, size());
+	recycle(d, size());
 
 	if (islonlat) {
 		if (vt == "points") {
-			return point_buffer(dist, quadsegs, false);
+			return point_buffer(d, quadsegs, false);
 		} else {
 			SpatVector p;
 			bool ispol = vt == "polygons";
@@ -1057,17 +1129,17 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 				if (ispol) {
 					SpatVector h = p.get_holes();
 					p = p.remove_holes();
-					p = lonlat_buf(p, dist[i], quadsegs, true, false);
-					if (h.size() > 0) {
-						h = lonlat_buf(h, dist[i], quadsegs, true, true);
-						if (h.size() > 0) {
+					p = lonlat_buf(p, d[i], quadsegs, true, false);
+					if (!h.empty()) {
+						h = lonlat_buf(h, d[i], quadsegs, true, true);
+						if (!h.empty()) {
 							for (size_t j=0; j<h.geoms[0].parts.size(); j++) {
 								p.geoms[0].parts[0].addHole(h.geoms[0].parts[j].x, h.geoms[0].parts[j].y);
 							}
 						}
 					}
 				} else {
-					p = lonlat_buf(p, dist[i], quadsegs, false, false);
+					p = lonlat_buf(p, d[i], quadsegs, false, false);
 				}
 				out = out.append(p, true);
 			}
@@ -1077,14 +1149,37 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 	}
 
 
-
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 //	SpatVector f = remove_holes();
 
+	GEOSBufferParams* bufparms = GEOSBufferParams_create_r(hGEOSCtxt);	
+	GEOSBufferParams_setQuadrantSegments_r(hGEOSCtxt, bufparms, quadsegs);
+	if (capstyle == "flat") {
+		GEOSBufferParams_setEndCapStyle_r(hGEOSCtxt, bufparms, GEOSBUF_CAP_FLAT);
+	} else if (capstyle == "square") { 
+		GEOSBufferParams_setEndCapStyle_r(hGEOSCtxt, bufparms, GEOSBUF_CAP_SQUARE);
+	} else {
+		GEOSBufferParams_setEndCapStyle_r(hGEOSCtxt, bufparms, GEOSBUF_CAP_ROUND);
+	}
+	if (joinstyle == "mitre") {
+		GEOSBufferParams_setJoinStyle_r(hGEOSCtxt, bufparms, GEOSBUF_JOIN_MITRE);
+	} else if (joinstyle == "bevel") { 
+		GEOSBufferParams_setJoinStyle_r(hGEOSCtxt, bufparms, GEOSBUF_JOIN_BEVEL);
+	} else {
+		GEOSBufferParams_setJoinStyle_r(hGEOSCtxt, bufparms, GEOSBUF_JOIN_ROUND);
+	}
+	if (!std::isnan(mitrelimit)) { 
+		GEOSBufferParams_setMitreLimit_r(hGEOSCtxt, bufparms, mitrelimit);
+	}
+	if (singlesided) { 
+		GEOSBufferParams_setSingleSided_r(hGEOSCtxt, bufparms, 1);
+	} 
+	
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> b(size());
 	for (size_t i = 0; i < g.size(); i++) {
-		GEOSGeometry* pt = GEOSBuffer_r(hGEOSCtxt, g[i].get(), dist[i], quadsegs);
+		GEOSGeometry* pt = GEOSBufferWithParams_r(hGEOSCtxt, g[i].get(), bufparms, d[i]);		
+//		GEOSGeometry* pt = GEOSBuffer_r(hGEOSCtxt, g[i].get(), d[i], quadsegs);
 		if (pt == NULL) {
 			out.setError("GEOS exception");
 			geos_finish(hGEOSCtxt);
@@ -1094,6 +1189,9 @@ SpatVector SpatVector::buffer(std::vector<double> dist, unsigned quadsegs) {
 	}
 	SpatVectorCollection coll = coll_from_geos(b, hGEOSCtxt);
 
+
+	GEOSBufferParams_destroy_r(hGEOSCtxt, bufparms);	
+
 //	out = spat_from_geom(hGEOSCtxt, g, "points");
 	geos_finish(hGEOSCtxt);
 	out = coll.get(0);
@@ -1169,7 +1267,7 @@ SpatVector SpatVector::intersect(SpatVector v, bool values) {
 
 
 	//SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
-		if (result.size() > 0) {
+		if (!result.empty()) {
 			SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids, true, false);
 			out = coll.get(0);
 			out.srs = srs;
@@ -1308,14 +1406,6 @@ int getRel(std::string &relation) {
 	return pattern;
 }
 
-
-
-void callbck(void *item, void *userdata) { // callback function for tree selection
-	std::vector<size_t> *ret = (std::vector<size_t> *) userdata;
-	ret->push_back(*((size_t *) item));
-}
-
-
 std::vector<int> SpatVector::relate(SpatVector v, std::string relation, bool prepared, bool index) {
 	// this method is redundant with "which_relate")
 	std::vector<int> out;
@@ -2094,7 +2184,7 @@ SpatVector SpatVector::mask(SpatVector x, bool inverse) {
 typedef int (* dist_fn)(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *, double *);
 
 bool get_dist_fun(dist_fn &f, std::string s) {
-	if ((s == "Euclidean") || (s == ""))
+	if ((s == "Euclidean") || s.empty())
 		f = GEOSDistance_r;
 	else if (s == "Hausdorff")
 		f = GEOSHausdorffDistance_r;
@@ -2416,7 +2506,7 @@ SpatVector SpatVector::erase_agg(SpatVector v) {
 			rids.push_back(i);
 		}
 	}
-	if (result.size() > 0) {
+	if (!result.empty()) {
 		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
 		out = coll.get(0);
 		out.srs = srs;
@@ -2475,7 +2565,10 @@ SpatVector SpatVector::erase(SpatVector v) {
 		if (good) rids.push_back(i);
 	}
 
-	if (rids.size() > 0) {
+	if (rids.empty()) {
+		std::vector<int> none(1, -1);
+		out = subset_rows(none);
+	} else {
 		SpatVectorCollection coll = coll_from_geos(x, hGEOSCtxt);
 		out = coll.get(0);
 		out.srs = srs;
@@ -2483,9 +2576,6 @@ SpatVector SpatVector::erase(SpatVector v) {
 		if (rids.size() != out.nrow()) {
 			out = out.subset_rows(rids);
 		}
-	} else {
-		std::vector<int> none(1, -1);
-		out = subset_rows(none);
 	}
 	geos_finish(hGEOSCtxt);
 	if (!srs.is_same(v.srs, true)) {
@@ -2666,7 +2756,7 @@ SpatVector SpatVector::gaps() {
 SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 	SpatVector out;
 
-	if ((size() == 0) || (v.size()==0)) {
+	if ((size() == 0) || v.empty()) {
 		out.setError("empty SpatVecor(s)");
 		return out;
 	}
