@@ -23,6 +23,8 @@
 #include <map>
 
 #include "vecmath.h"
+#include "vecmathse.h"
+
 #include "math_utils.h"
 #include "string_utils.h"
 
@@ -172,8 +174,8 @@ SpatRaster SpatRaster::quantile(std::vector<double> probs, bool narm, SpatOption
 		return out;
 	}
 
-	double pmin = vmin(probs, false);
-	double pmax = vmin(probs, false);
+	double pmin = min_se(probs, 0, probs.size());
+	double pmax = max_se(probs, 0, probs.size());
 	if ((std::isnan(pmin)) || (std::isnan(pmax)) || (pmin < 0) || (pmax > 1)) {
 		SpatRaster out = geometry(1);
 		out.setError("intvalid probs");
@@ -532,10 +534,26 @@ SpatDataFrame SpatRaster::zonal(SpatRaster z, std::string fun, bool narm, SpatOp
 	}
 
 	if (z.nlyr() > 1) {
+		// this is not very efficient. Should deal with multiple z layers below.
 		SpatOptions xopt(opt);
-		std::vector<unsigned> lyr = {0};
-		z = z.subset(lyr, xopt);
-		out.addWarning("only the first zonal layer is used");
+		SpatDataFrame spout;
+		std::vector<std::string> nms = z.getNames();
+		make_unique_names(nms);
+		z.setNames(nms);
+		
+		for (unsigned i=0; i<z.nlyr(); i++) {
+			std::vector<unsigned> lyr = {i};
+			SpatRaster zz = z.subset(lyr, xopt);
+			SpatDataFrame spd = zonal(zz, fun, narm, xopt);
+			std::vector<long> id(spd.nrow(), i);
+			spd.add_column(id, "zlyr");
+			if (i == 0) {
+				spout = spd;
+			} else {
+				spout.rbind(spd);
+			}
+		}
+		return spout;
 	}
 
 	double posinf = std::numeric_limits<double>::infinity();
@@ -763,3 +781,341 @@ SpatDataFrame SpatRaster::zonal(SpatRaster z, std::string fun, bool narm, SpatOp
 //	}
 	return(out);
 }
+
+
+
+
+SpatDataFrame SpatRaster::zonal_weighted(SpatRaster z, SpatRaster w, bool narm, SpatOptions &opt) {
+
+	SpatDataFrame out;
+	if (!hasValues()) {
+		out.setError("SpatRaster has no values");
+		return(out);
+	}
+	if (!z.hasValues()) {
+		out.setError("zonal SpatRaster has no values");
+		return(out);
+	}
+	if (!w.hasValues()) {
+		out.setError("weights SpatRaster has no values");
+		return(out);
+	}
+	if (!compare_geom(z, false, true, opt.get_tolerance(), true)) {
+		out.setError(getError());
+		return(out);
+	}
+	if (!compare_geom(w, false, true, opt.get_tolerance(), true)) {
+		out.setError(getError());
+		return(out);
+	}
+	if (hasWarning()) {
+		std::vector<std::string> w = getWarnings();
+		for (size_t i=0; i<w.size(); i++) {
+			out.addWarning(w[i]);
+		}
+	}
+
+	if (z.nlyr() > 1) {
+		SpatOptions xopt(opt);
+		std::vector<unsigned> lyr = {0};
+		z = z.subset(lyr, xopt);
+		out.addWarning("only the first zonal layer is used");
+	}
+	if (w.nlyr() > 1) {
+		SpatOptions xopt(opt);
+		std::vector<unsigned> lyr = {0};
+		w = w.subset(lyr, xopt);
+		out.addWarning("only the first weights layer is used");
+	}
+
+	if (!readStart()) {
+		out.setError(getError());
+		return(out);
+	}
+	if (!z.readStart()) {
+		out.setError(z.getError());
+		return(out);
+	}
+	if (!w.readStart()) {
+		out.setError(z.getError());
+		return(out);
+	}
+	opt.ncopies = 8;
+	BlockSize bs = getBlockSize(opt);
+
+	size_t nl = nlyr();
+	size_t nc = ncol();
+	std::vector<std::map<double, double>> m(nl);
+	std::vector<std::map<double, size_t>> wsum(nl);
+
+	for (size_t i=0; i<bs.n; i++) {
+		unsigned nrc = bs.nrows[i] * nc;
+		std::vector<double> vv, zv, wv;
+		readValues(vv, bs.row[i], bs.nrows[i], 0, ncol());
+		z.readValues(zv, bs.row[i], bs.nrows[i], 0, ncol());
+		w.readValues(wv, bs.row[i], bs.nrows[i], 0, ncol());
+
+		for (size_t j=0; j<nl; j++) {
+			size_t off = j*nrc;
+			std::vector<double> v(vv.begin()+off, vv.begin() + off + nrc);
+			for (size_t k=0; k<nrc; k++) {
+				if (std::isnan(zv[k]) || std::isnan(wv[k]) || (wv[k] < 0)) {
+					continue;
+				}
+				if (narm && std::isnan(v[k])) {
+					if (m[j].find(zv[k]) == m[j].end()) {
+						m[j][zv[k]] = 0;
+						wsum[j][zv[k]] = 0;
+					}
+				} else if (m[j].find(zv[k]) == m[j].end()) {
+					m[j][zv[k]] = v[k] * wv[k];
+					wsum[j][zv[k]] = wv[k];
+				} else {
+					m[j][zv[k]] += v[k] * wv[k];
+					wsum[j][zv[k]] += wv[k];
+				}
+			}
+		}
+	}
+
+	readStop();
+	z.readStop();
+	w.readStop();
+
+	std::vector<double> zone;
+	std::vector<std::string> nms = getNames();
+
+	size_t n = m[0].size();
+	zone.reserve(n);
+	for (size_t i=0; i<nl; i++) {
+		std::vector<double> value;
+		value.reserve(n);
+		if (i==0) {
+			for (auto& it : m[0]) {
+				zone.push_back(it.first);
+				value.push_back(it.second);
+			}
+			out.add_column(zone, "zone");
+		} else {
+			for (auto& it : m[i]) {
+				value.push_back(it.second);
+			}
+		}
+		size_t j = 0;
+		for (auto& it : wsum[i]) {
+			double d = (double)it.second;
+			if (d > 0) {
+				value[j] /= d;
+			} else {
+				value[j] = NAN;
+			}
+			j++;
+		}
+		out.add_column(value, nms[i]);
+	}
+	return(out);
+}
+
+
+SpatDataFrame SpatRaster::zonal_poly(SpatVector x, std::string fun, bool weights, bool exact, bool touches,bool narm, SpatOptions &opt) {
+
+	SpatDataFrame out;
+	std::string gtype = x.type();
+	if (gtype != "polygons") {
+		out.setError("SpatVector must have polygon geometry");
+		return out;
+	}
+	
+	if (!hasValues()) {
+		out.setError("raster has no values");
+		return out;
+	}
+
+	if ((weights || exact)) {
+		if ((fun != "mean") && (fun!="min") && (fun!="max")) {
+			out.setError("fun should be 'min', 'max' or 'mean' when using weights/exact");
+			return out;			
+		}
+	}
+
+	if (!haveseFun(fun)) {
+		out.setError("Unknown function");
+		return out;
+	}
+	std::function<double(std::vector<double>&, double, double)> zfun= getseFun(fun, narm);
+
+    unsigned nl = nlyr();
+    unsigned ng = x.size();
+
+	std::vector<std::vector<double>> zv(nl, std::vector<double>(ng));
+	
+    SpatRaster r = geometry(1);
+    for (size_t i=0; i<ng; i++) {
+		SpatGeom g = x.getGeom(i);
+		SpatVector p(g);
+		p.srs = x.srs;
+		std::vector<double> cell, wgt;
+		if (weights) {
+			rasterizeCellsWeights(cell, wgt, p, opt);
+		} else if (exact) {
+			rasterizeCellsExact(cell, wgt, p, opt);
+		} else {
+			cell = rasterizeCells(p, touches, opt);
+        }
+		
+		std::vector<std::vector<double>> e = extractCell(cell);
+ 		if ((weights || exact) && fun == "mean") {
+			if (narm) {
+				for (size_t j=0; j<nl; j++) {
+					double wsum = 0;
+					double vsum = 0;
+					for (size_t k=0; k<e[j].size(); k++) {
+						if (!std::isnan(e[j][k])) {
+							wsum += wgt[k];
+							vsum += (e[j][k] * wgt[k]);  
+						}
+					}
+					zv[j][i] = vsum / wsum;
+				}
+			} else {
+				for (size_t j=0; j<nl; j++) {
+					double wsum = 0;
+					double vsum = 0;
+					for (size_t k=0; k<e[j].size(); k++) {
+						wsum += wgt[k];
+						vsum += (e[j][k] * wgt[k]);  
+					}
+					zv[j][i] = vsum / wsum;
+				}
+			}
+		} else {
+			for (size_t j=0; j<nl; j++) {
+				zv[j][i] = zfun(e[j], 0, e[j].size());
+			}
+		}
+	}
+	std::vector<std::string> nms = getNames();	
+	for (size_t j=0; j<nl; j++) {
+		out.add_column(zv[j], nms[j]);
+	}
+	
+	return out;
+}
+
+
+SpatDataFrame SpatRaster::zonal_poly_weighted(SpatVector x, SpatRaster w, bool weights, bool exact, bool touches, bool narm, SpatOptions &opt) {
+
+	SpatDataFrame out;
+	std::string gtype = x.type();
+	if (gtype != "polygons") {
+		out.setError("SpatVector must have polygon geometry");
+		return out;
+	}
+	
+	if (!compare_geom(w, false, true, opt.get_tolerance(), true)) {
+		out.setError(getError());
+		return(out);
+	}
+	if (!hasValues()) {
+		out.setError("raster has no values");
+		return out;
+	}
+	if (!w.hasValues()) {
+		out.setError("raster has no values");
+		return out;
+	}
+
+    unsigned nl = nlyr();
+    unsigned ng = x.size();
+
+	std::vector<std::vector<double>> zv(nl, std::vector<double>(ng));
+	
+    SpatRaster r = geometry(1);
+    for (size_t i=0; i<ng; i++) {
+		SpatGeom g = x.getGeom(i);
+		SpatVector p(g);
+		p.srs = x.srs;
+		std::vector<double> cell, wgt;
+		if (weights) {
+			rasterizeCellsWeights(cell, wgt, p, opt);
+		} else if (exact) {
+			rasterizeCellsExact(cell, wgt, p, opt);
+		} else {
+			cell = rasterizeCells(p, touches, opt);
+        }
+		
+		std::vector<std::vector<double>> e = extractCell(cell);
+		std::vector<std::vector<double>> we = w.extractCell(cell);
+		
+ 		if (weights || exact) {
+			if (narm) {
+				for (size_t j=0; j<nl; j++) {
+					double wsum = 0;
+					double vsum = 0;
+					for (size_t k=0; k<e[j].size(); k++) {
+						if (!std::isnan(e[j][k])) {
+							wsum += we[0][k] * wgt[k];
+							vsum += (e[j][k] * we[0][k] * wgt[k]);  
+						}
+					}
+					zv[j][i] = vsum / wsum;
+				}
+				for (size_t j=0; j<nl; j++) {
+					double wsum = 0;
+					double vsum = 0;
+					for (size_t k=0; k<e[j].size(); k++) {
+						if ((!std::isnan(e[j][k])) && (!std::isnan(we[0][k]))) {
+							wsum += we[0][k] * wgt[k];
+							vsum += (e[j][k] * we[0][k] * wgt[k]);  
+						}
+					}
+					zv[j][i] = vsum / wsum;
+				}				
+
+
+			} else {
+				for (size_t j=0; j<nl; j++) {
+					double wsum = 0;
+					double vsum = 0;
+					for (size_t k=0; k<e[j].size(); k++) {
+						wsum += we[0][k];
+						vsum += (e[j][k] * we[0][k]);  
+					}
+					zv[j][i] = vsum / wsum;
+				}
+			}
+		} else {
+			if (narm) {
+				for (size_t j=0; j<nl; j++) {
+					double wsum = 0;
+					double vsum = 0;
+					for (size_t k=0; k<e[j].size(); k++) {
+						if ((!std::isnan(e[j][k])) && (!std::isnan(we[0][k]))) {
+							wsum += we[0][k];
+							vsum += (e[j][k] * we[0][k]);  
+						}
+					}
+					zv[j][i] = vsum / wsum;
+				}				
+			} else {
+				for (size_t j=0; j<nl; j++) {
+					double wsum = 0;
+					double vsum = 0;
+					for (size_t k=0; k<e[j].size(); k++) {
+						wsum += we[0][k];
+						vsum += (e[j][k] * we[0][k]);  
+					}
+					zv[j][i] = vsum / wsum;
+				}
+			}
+		}
+	}
+	std::vector<std::string> nms = getNames();	
+	for (size_t j=0; j<nl; j++) {
+		out.add_column(zv[j], nms[j]);
+	}
+	
+	return out;
+}
+
+
