@@ -52,7 +52,7 @@ void SpatRaster::gdalogrproj_init(std::string path) {
     OGRRegisterAll();
 	//GDALregistred = true;
 #if GDAL_VERSION_MAJOR >= 3
-	if (path != "") {
+	if (!path.empty()) {
 		const char *cp = path.c_str();
 		proj_context_set_search_paths(PJ_DEFAULT_CTX, 1, &cp);
 	}
@@ -113,7 +113,7 @@ bool read_aux_json(std::string filename, std::vector<int_64> &time, std::string
 				time = tm;
 			}
 		}
-		if ((istep >= 0) && (time.size() > 0)) {
+		if ((istep >= 0) && !time.empty()) {
 			std::vector<std::string> x = strsplit_first(s[istep], ":");
 			if (x.size() == 2) {
 				x = strsplit(x[1], ",");
@@ -155,20 +155,21 @@ SpatCategories GetRAT(GDALRasterAttributeTable *pRAT) {
 	size_t nc = (int) pRAT->GetColumnCount();
 	size_t nr = (int) pRAT->GetRowCount();
 
-
 	std::vector<std::string> ss = {"histogram", "count", "red", "green", "blue", "opacity", "r", "g", "b", "alpha"};
 
 	//std::vector<std::string> ratnms;
 	std::vector<int> id, id2;
 
 	bool hasvalue=false;
+	//std::string valuename = "";
 	for (size_t i=0; i<nc; i++) {
 		std::string name = pRAT->GetNameOfCol(i);
 		lowercase(name);
 		//ratnms.push_back(name);
-		if (name == "value") {
+		if (!hasvalue && ((name == "value") || (name == "id"))) {
 			id.insert(id.begin(), i);
 			hasvalue = true;
+			//valuename = name;
 		} else {
 			int k = where_in_vector(name, ss, true);
 			if (k >= 0) {
@@ -178,13 +179,12 @@ SpatCategories GetRAT(GDALRasterAttributeTable *pRAT) {
 			}
 		}
 	}
-	if ((id.size() <= 1) && (id2.size() == 1)) {
+	if (id.size() < 1) {
 // #790 avoid having just "count" or "histogram" 
 		return(out);
 	}
 	id.insert(id.end(), id2.begin(), id2.end());
 
-
 	if (!hasvalue) {
 		std::vector<long> vid(nr);
 		std::iota(vid.begin(), vid.end(), 0);
@@ -338,6 +338,7 @@ bool setIntCol(SpatDataFrame &d, SpatDataFrame &out, int k, std::string name) {
 
 bool colsFromRat(SpatDataFrame &d, SpatDataFrame &out) {
 
+
 	std::vector<std::string> ss = d.get_names();
 	for (size_t i=0; i<ss.size(); i++) {
 		lowercase(ss[i]);
@@ -346,9 +347,10 @@ bool colsFromRat(SpatDataFrame &d, SpatDataFrame &out) {
 	int k = where_in_vector("value", ss, true);
 	if (k >= 0) {
 		size_t j = d.iplace[k];
+		
 		if (d.itype[k] == 1) {
 			out.add_column(d.iv[j], "value");
-		} else {
+		} else if (d.itype[k] == 0) {
 			std::vector<long> x;
 			x.reserve(d.nrow());
 			for (size_t i=0; i<d.nrow(); i++) {
@@ -360,39 +362,48 @@ bool colsFromRat(SpatDataFrame &d, SpatDataFrame &out) {
 		return false;
 	}
 
-	std::vector<std::string> cols1 = {"red", "green", "blue"};
-	std::vector<std::string> cols2 = {"r", "g", "b"};
-	for (size_t i=0; i<3; i++) {
-		int k = where_in_vector(cols1[i], ss, true);
-		if (k >= 0) {
-			if (!setIntCol(d, out, k, cols1[i])) return false;
-		} else {
-			int k = where_in_vector(cols2[i], ss, true);
+
+//	int k = where_in_vector("colors", ss, true);
+//	if (k >= 0) {
+		//col2rgb(d, out, k);
+//	} else {
+		std::vector<std::string> cols1 = {"red", "green", "blue"};
+		std::vector<std::string> cols2 = {"r", "g", "b"};
+		for (size_t i=0; i<3; i++) {
+			int k = where_in_vector(cols1[i], ss, true);
 			if (k >= 0) {
 				if (!setIntCol(d, out, k, cols1[i])) return false;
 			} else {
-				return false;
+				int k = where_in_vector(cols2[i], ss, true);
+				if (k >= 0) {
+					if (!setIntCol(d, out, k, cols1[i])) return false;
+				} else {
+					return false;
+				}
 			}
 		}
-	}
 
-	bool have_alpha = false;
-	k = where_in_vector("alpha", ss, true);
-	if (k >= 0) {
-		if (setIntCol(d, out, k, "alpha")) have_alpha = true;
-	}
-	if (!have_alpha) {
-		int k = where_in_vector("transparency", ss, true);
+		bool have_alpha = false;
+		k = where_in_vector("alpha", ss, true);
 		if (k >= 0) {
 			if (setIntCol(d, out, k, "alpha")) have_alpha = true;
 		}
-	}
-	if (!have_alpha) {
-		std::vector<long> a(out.nrow(), 255);
-		out.add_column(a, "alpha");
-	}
 
+		if (!have_alpha) {
+			int k = where_in_vector("transparency", ss, true);
+			if (k >= 0) {
+				if (setIntCol(d, out, k, "alpha")) have_alpha = true;
+			}
+		}
+
+		if (!have_alpha) {
+			std::vector<long> a(out.nrow(), 255);
+			out.add_column(a, "alpha");
+		}
+//	}
+	
 	d = out;
+
 	return true;
 }
 
@@ -435,14 +446,14 @@ SpatCategories GetCategories(char **pCat, std::string name) {
 	for (long i = 0; i<n; i++) {
 		const char *field = CSLGetField(pCat, i);
 		std::string s = field;
-		if (s != "") {
+		if (!s.empty()) {
 			id.push_back(i);
 			nms.push_back(field);
 		}
 	}
 
 	scat.d.add_column(id, "value");
-	name = name == "" ? "category" : name;
+	name = name.empty() ? "category" : name;
 	scat.d.add_column(nms, name);
 	scat.index = 1;
 	return(scat);
@@ -461,7 +472,7 @@ std::string basename_sds(std::string f) {
 		f.erase(0, i + 1);
 	}
 	// this may be incorrect of the variable name includes a ":" ?
-	const size_t j = f.find_last_of(":");
+	const size_t j = f.find_last_of(':');
 	if (std::string::npos != j) {
 		f.erase(0, j + 1);
 	}
@@ -604,7 +615,7 @@ SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool u
 				s.erase(0, pos + delim.length());
 				SpatRaster sub;
 				if (sub.constructFromFile(s, {-1}, {""}, {}, {})) {
-					std::string sname = (sub.source[0].source_name != "") ? sub.source[0].source_name : basename_sds(s);
+					std::string sname = sub.source[0].source_name.empty() ? basename_sds(s) : sub.source[0].source_name;
 					if (!push_back(sub, sname, sub.source[0].source_name_long, sub.source[0].unit[0], true)) {
 						addWarning("skipped (different geometry): " + s);
 					}
@@ -706,6 +717,41 @@ SpatRaster SpatRaster::fromFiles(std::vector<std::string> fname, std::vector<int
 
 */
 
+bool getGCPs(GDALDataset *poDataset, SpatRasterSource &s) {
+	int n = poDataset->GetGCPCount();
+//	Rcpp::Rcout << "n GCP " << n << std::endl;
+	if (n == 0) return false;
+	const GDAL_GCP *gcp;
+	gcp	= poDataset->GetGCPs();
+	
+	double adfGeoTransform[6];
+	if (GDALGCPsToGeoTransform(n, gcp, adfGeoTransform, true)) {
+		//for (size_t i=0; i<6; i++) {
+		//	Rcpp::Rcout << adfGeoTransform[i] << " ";
+		//}
+		//Rcpp::Rcout << std::endl;
+		double xmin = adfGeoTransform[0]; /* left x */
+		double xmax = xmin + adfGeoTransform[1] * s.ncol; /* w-e resolution */
+		if (xmin > xmax) {
+			std::swap(xmin, xmax);			
+		}
+		double ymax = adfGeoTransform[3]; // top y
+		double ymin = ymax + s.nrow * adfGeoTransform[5];
+		if (adfGeoTransform[5] > 0) {
+			s.flipped = true;
+			std::swap(ymin, ymax);
+		}
+		SpatExtent e(xmin, xmax, ymin, ymax);
+		s.extent = e;
+		if (adfGeoTransform[2] != 0 || adfGeoTransform[4] != 0) {
+			s.rotated = true;
+		}
+		return true;
+	}
+	return false;
+}
+
+
 bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> drivers, std::vector<std::string> options) {
 
     GDALDataset *poDataset = openGDAL(fname, GDAL_OF_RASTER | GDAL_OF_READONLY | GDAL_OF_VERBOSE_ERROR, drivers, options);
@@ -736,8 +782,12 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		return false;
 	}
 
-
 	SpatRasterSource s;
+
+	char **metasrc = poDataset->GetMetadata();
+	while (metasrc != nullptr && *metasrc != nullptr) {
+		s.smdata.push_back(*metasrc++);
+	}
 	s.ncol = poDataset->GetRasterXSize();
 	s.nrow = poDataset->GetRasterYSize();
 	s.nlyr = nl;
@@ -770,7 +820,12 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			s.rotated = true;
 			addWarning("the data in this file are rotated. Use 'rectify' to fix that");
 		}
+	} else if (getGCPs(poDataset, s)) {
+		if (s.rotated) {
+			addWarning("the data in this file are rotated. Use 'rectify' to fix that");
+		}
 	} else {
+
 		hasExtent = false;
 		SpatExtent e(0, s.ncol, 0, s.nrow);
 		s.extent = e;
@@ -804,7 +859,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 */
 
 	std::string crs = getDsWKT(poDataset);
-	if (crs == "") {
+	if (crs.empty()) {
 		if (hasExtent && s.extent.xmin >= -180 && s.extent.xmax <= 360 && s.extent.ymin >= -90 && s.extent.ymax <= 90) {
 			crs = "OGC:CRS84";
 			s.parameters_changed = true;
@@ -826,12 +881,12 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		units.resize(0);
 		addWarning("could not parse aux.json");
 	}
-	if (timestamps.size() > 0) {
+	if (!timestamps.empty()) {
 		s.time = timestamps;
 		s.timestep = timestep;
 		s.hasTime = true;
 	}
-	if (units.size() > 0) {
+	if (!units.empty()) {
 		s.unit = units;
 		s.hasUnit = true;
 	}
@@ -853,9 +908,9 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	for (size_t i = 0; i < s.nlyr; i++) {
 		poBand = poDataset->GetRasterBand(i+1);
 
-		if ((gdrv=="netCDF") || (gdrv == "HDF5") || (gdrv == "GRIB")) {
+		if ((gdrv=="netCDF") || (gdrv == "HDF5") || (gdrv == "GRIB") || (gdrv == "GTiff")) {
 			char **m = poBand->GetMetadata();
-			while (*m != nullptr) {
+			while (m != nullptr && *m != nullptr) {
 				bandmeta[i].push_back(*m++);
 			}
 		}
@@ -973,8 +1028,9 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 				nm = nms[s.cats[i].index];
 			}
 		}
-		if (nm == "") {
-			if (bandname != "") {
+		
+		if (nm.empty()) {
+			if (!bandname.empty()) {
 				nm = bandname;
 			} else if (s.nlyr > 1) {
 				nm = s.source_name + "_" + std::to_string(i+1);
@@ -1001,9 +1057,13 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			}
 		}
 		s.set_names_time_ncdf(metadata, bandmeta, msg);
-	} else if (gdrv == "GRIB") {
+	} else if (gdrv == "GRIB") {	
 		s.set_names_time_grib(bandmeta, msg);
+	} else if (gdrv == "GTiff") {	
+	// needs to get its own generic one 
+		s.set_names_time_tif(bandmeta, msg);
 	}
+	s.bmdata = bandmeta;
 	if (msg.size() > 1) {
 		addWarning(msg);
 	}
@@ -1112,7 +1172,7 @@ void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t r
 		return;
 	}
 
-	if (!source[src].open_read) {
+	if (!(source[src].open_read || source[src].open_write)) {
 		setError("the file is not open for reading");
 		return;
 	}
@@ -1132,10 +1192,10 @@ void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t r
 		}
 	}
 
-	if (panBandMap.size() > 0) {
-		err = source[src].gdalconnection->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
-	} else {
+	if (panBandMap.empty()) {
 		err = source[src].gdalconnection->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
+	} else {
+		err = source[src].gdalconnection->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 	}
 
 	GDALRasterBand  *poBand;
@@ -1219,10 +1279,10 @@ std::vector<double> SpatRaster::readValuesGDAL(unsigned src, size_t row, size_t
 	int hasNA;
 	std::vector<double> naflags(nl, NAN);
 	CPLErr err = CE_None;
-	if (panBandMap.size() > 0) {
-		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
-	} else {
+	if (panBandMap.empty()) {
 		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
+	} else {
+		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], ncols, nrows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 	}
 
 	if (err == CE_None ) {
@@ -1292,10 +1352,10 @@ std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_
 	}
 */
 
-	if (panBandMap.size() > 0) {
-		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], scols, srows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
-	} else {
+	if (panBandMap.empty()) {
 		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], scols, srows, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
+	} else {
+		err = poDataset->RasterIO(GF_Read, col, row, ncols, nrows, &out[0], scols, srows, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 	}
 
 	if (err == CE_None ) {
@@ -1375,10 +1435,10 @@ std::vector<std::vector<double>> SpatRaster::readRowColGDAL(unsigned src, std::v
 	CPLErr err = CE_None;
 	for (size_t j=0; j < n; j++) {
 		if ((cols[j] < 0) || (rows[j] < 0)) continue;
-		if (panBandMap.size() > 0) {
-			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
-		} else {
+		if (panBandMap.empty()) {
 			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
+		} else {
+			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 		}
 		if (err != CE_None ) {
 			break;
@@ -1456,10 +1516,10 @@ std::vector<double> SpatRaster::readRowColGDALFlat(unsigned src, std::vector<int
 	CPLErr err = CE_None;
 	for (size_t j=0; j < n; j++) {
 		if ((cols[j] < 0) || (rows[j] < 0)) continue;
-		if (panBandMap.size() > 0) {
-			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
-		} else {
+		if (panBandMap.empty()) {
 			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
+		} else {
+			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 		}
 		if (err != CE_None ) {
 			break;
@@ -1542,7 +1602,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 	std::vector<std::vector<std::string>> info = parse_metadata_sds(meta);
 	int n = info[0].size();
 
-	if (gtiff && (subds[0] < 0) && (subdsname[0] == "")) {
+	if (gtiff && (subds[0] < 0) && subdsname[0].empty()) {
 		subds.resize(n);
 		std::iota(subds.begin(), subds.end(), 0);
 	}
@@ -1551,11 +1611,11 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 // std::vector<unsigned> varnl;
 // for selection based on nlyr
 
-	if (info[0].size() == 0) {
+	if (info[0].empty()) {
 		return false;
 	}
 	// select sds by index
-	if (((subds.size() > 0) && (subds[0] >= 0))) {
+	if ((!subds.empty() && (subds[0] >= 0))) {
 		for (size_t i=0; i<subds.size(); i++) {
 			if (subds[i] >=0 && subds[i] < n) {
 				sd.push_back(info[0][subds[i]]);
@@ -1567,7 +1627,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 			}
 		}
 	// select by name
-	} else if ((subdsname.size() > 0) && (subdsname[0] != "")) {
+	} else if (!subdsname.empty() && !subdsname[0].empty()) {
 		for (size_t i=0; i<subdsname.size(); i++) {
 			int w = where_in_vector(subdsname[i], info[1], false);
 			if (w >= 0) {
@@ -1594,7 +1654,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 				cols.push_back(stoi(info[4][i]));
 			}
 		}
-		if (sd.size() == 0) { // all were removed
+		if (sd.empty()) { // all were removed
 			std::vector<size_t> nl(n);
 			for (size_t i=0; i<nl.size(); i++) {
 				nl[i] = stol(info[5][i]);
@@ -1646,7 +1706,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 		}
 	}
 
-	if (skipped.size() > 0) {
+	if (!skipped.empty()) {
 		std::string s="skipped sub-datasets (see 'describe(sds=TRUE)'):\n" + skipped[0];
 		for (size_t i=1; i<skipped.size(); i++) {
 			s += ", " + skipped[i];
@@ -1663,7 +1723,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 			make_unique_names(nms);
 			lyrnames.insert(lyrnames.end(), nms.begin(), nms.end());
 		}
-		if (lyrnames.size() > 0) {
+		if (!lyrnames.empty()) {
 			setNames(lyrnames, false);
 		}
 	}
@@ -1709,7 +1769,7 @@ bool get_double(std::string input, double &output) {
 std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vector<std::string> vals, std::string &step, std::string &msg) {
 
 	std::vector<int_64> out, bad;
-	if (vals.size() < 1) {
+	if (vals.empty()) {
 		step = "";
 		return out;
 	}
@@ -1753,31 +1813,37 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 	}
 
 	bool years = false;
+	bool yearsbp = false;
 	bool days = false;
 	bool hours = false;
 	bool seconds = false;
 	bool foundorigin = false;
 
 	if (fu) {
+		lowercase(origin);
 		if ((origin.find("hours")) != std::string::npos) {
 			hours = true;
 		} else if ((origin.find("days")) != std::string::npos) {
 			days = true;
 		} else if ((origin.find("seconds")) != std::string::npos) {
 			seconds = true;
+		} else if ((origin.find("years before present")) != std::string::npos) {
+			yearsbp = true;
+			foundorigin = true;
 		} else if ((origin.find("years")) != std::string::npos) {
 			years = true;
 		}
-		size_t pos;
-		if ((pos = origin.find("from")) != std::string::npos) {
-			origin.erase(0, pos + 5);
-			foundorigin = true;
-		} else if ((pos = origin.find("since")) != std::string::npos) {
-			origin.erase(0, pos + 6);
-			foundorigin = true;
+		if (!foundorigin) {
+			size_t pos;		
+			if ((pos = origin.find("from")) != std::string::npos) {
+				origin.erase(0, pos + 5);
+				foundorigin = true;
+			} else if ((pos = origin.find("since")) != std::string::npos) {
+				origin.erase(0, pos + 6);
+				foundorigin = true;
+			}
 		}
 	}
-
 	SpatTime_t offset = 0;
 	if (foundorigin) {
 		step = "seconds";
@@ -1807,6 +1873,10 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 			step = "years";
 			int syear = getyear(origin);
 			for (size_t i=0; i<raw.size(); i++) out.push_back(get_time(syear+raw[i], 6, 30, 0, 0, 0));
+		} else if (yearsbp) {
+			step = "years";
+			int syear = 1950;
+			for (size_t i=0; i<raw.size(); i++) out.push_back(get_time(syear+raw[i], 6, 30, 0, 0, 0));
 		} else {
 			step = "raw";
 			for (size_t i=0; i<raw.size(); i++) out.push_back(raw[i]);
@@ -1825,7 +1895,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 std::vector<std::vector<std::string>> ncdf_names(const std::vector<std::vector<std::string>> &m) {
 
 	std::vector<std::vector<std::string>> out(3);
-	if (m.size() < 1) return out;
+	if (m.empty()) return out;
 
 	std::string vname, lname, units = "";
 	std::vector<std::string> b = m[0];
@@ -1847,7 +1917,7 @@ std::vector<std::vector<std::string>> ncdf_names(const std::vector<std::vector<s
 		}
 		pos = b[j].find("standard_name=");
 		if (pos != std::string::npos) {
-			if (lname == "") {
+			if (lname.empty()) {
 				lname = b[j].erase(0, pos+14);
 			}
 		}
@@ -1877,24 +1947,24 @@ std::vector<std::vector<std::string>> ncdf_names(const std::vector<std::vector<s
 
 void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, std::vector<std::vector<std::string>> bandmeta, std::string &msg) {
 
-	if (bandmeta.size() == 0) return;
+	if (bandmeta.empty()) return;
 	std::vector<std::vector<std::string>> nms = ncdf_names(bandmeta);
 
-	if (nms[1].size() > 0) {
+	if (!nms[1].empty()) {
 		names = nms[1];
 		make_unique_names(names);
 	}
 	source_name = nms[2][0];
 	source_name_long = nms[2][1];
 
-	if (nms[2][2].size() == 0) {
+	if (nms[2][2].empty()) {
 		unit = {""};
 	} else {
 		unit = {nms[2][2]};
 	}
 
 	recycle(unit, nlyr);
-	if (nms[0].size() > 0) {
+	if (!nms[0].empty()) {
 		std::string step;
 		std::vector<int_64> x = ncdf_time(metadata, nms[0], step, msg);
 		if (x.size() == nlyr) {
@@ -1911,7 +1981,7 @@ void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, st
 std::vector<std::vector<std::string>> grib_names(const std::vector<std::vector<std::string>> &m) {
 
 	std::vector<std::vector<std::string>> out(4);
-	if (m.size() < 1) return out;
+	if (m.empty()) return out;
 	
 	bool ft1 = false;
 	bool ft2 = false;
@@ -1981,7 +2051,7 @@ std::vector<std::vector<std::string>> grib_names(const std::vector<std::vector<s
 
 void SpatRasterSource::set_names_time_grib(std::vector<std::vector<std::string>> bandmeta, std::string &msg) {
 
-	if (bandmeta.size() == 0) return;
+	if (bandmeta.empty()) return;
 	
 	std::vector<std::vector<std::string>> nms = grib_names(bandmeta);
 
@@ -2003,7 +2073,7 @@ void SpatRasterSource::set_names_time_grib(std::vector<std::vector<std::string>>
 		hastime = true;
 		int_64 tim;
 		for (size_t i=0; i<nms[2].size(); i++) {
-			if (nms[2][i] == "") {
+			if (nms[2][i].empty()) {
 				hastime = false;
 				break;
 			}
@@ -2025,3 +2095,91 @@ void SpatRasterSource::set_names_time_grib(std::vector<std::vector<std::string>>
 	
 }
 
+
+
+std::vector<std::vector<std::string>> tiff_names(const std::vector<std::vector<std::string>> &m) {
+
+	std::vector<std::vector<std::string>> out(4);
+	if (m.empty()) return out;
+	
+	for (size_t i=0; i<m.size(); i++) {
+
+		std::string time, units = "";
+		bool keepgoing = false;
+
+		for (size_t j=0; j<m[i].size(); j++) {			
+			size_t pos = m[i][j].find("UNIT=");
+			if (pos == std::string::npos) {
+				pos = m[i][j].find("unit=");
+			}	
+			if (pos != std::string::npos) {
+				units = m[i][j];
+				units.erase(0, pos+5);
+				str_replace(units, "[", "");
+				str_replace(units, "]", "");
+				lrtrim(units);
+				keepgoing = true;
+				continue;
+			}
+			pos = m[i][j].find("TIME=");
+			if (pos == std::string::npos) {
+				pos = m[i][j].find("time=");
+			}	
+			if (pos != std::string::npos) {
+				std::string tmp = m[i][j];
+				tmp.erase(0, pos+5);
+				pos = tmp.find("sec");
+				if (pos != std::string::npos) {
+					tmp.erase(tmp.begin()+pos, tmp.end());
+				}
+				time = tmp;
+				keepgoing = true;
+			}
+			if (!keepgoing) return out;
+		}
+		out[1].push_back(units);
+		out[2].push_back(time);
+	}
+	
+	return out;
+}
+
+
+void SpatRasterSource::set_names_time_tif(std::vector<std::vector<std::string>> bandmeta, std::string &msg) {
+
+	if (bandmeta.empty()) return;
+	
+	std::vector<std::vector<std::string>> nms = tiff_names(bandmeta);
+
+	if (nms[1].size() == nlyr) {
+		unit = {nms[0]};
+	}
+
+	bool hastime = false;
+	std::vector<int_64> tm;
+	if (nms[1].size() == nlyr) {
+		hastime = true;
+		int_64 tim;
+		for (size_t i=0; i<nms[1].size(); i++) {
+			if (nms[1][i].empty()) {
+				hastime = false;
+				break;
+			}
+			try {
+				tim = stol(nms[1][i]);
+			} catch(...) {
+				hastime = false;
+				break;
+			}
+			tm.push_back(tim);
+		}
+	}
+
+	if (hastime) {
+		time = tm;
+		timestep = "seconds";
+		hasTime = true;
+	}
+	
+}
+
