@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2022  Robert J. Hijmans
+// Copyright (c) 2018-2023  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -38,7 +38,8 @@ class SpatHole {
 		SpatHole();
 		SpatHole(std::vector<double> X, std::vector<double> Y);
 		//methods
-		size_t size() { return x.size(); }	
+		size_t size() { return x.size(); }
+		bool empty() { return x.empty(); }
 };
 
 class SpatPart {
@@ -55,11 +56,12 @@ class SpatPart {
 
 		//methods
 		size_t size() { return x.size(); }
+		bool empty() { return x.empty(); }
 		//holes, polygons only
 		bool addHole(std::vector<double> X, std::vector<double> Y);
 		bool addHole(SpatHole h);
 		SpatHole getHole(unsigned i) { return( holes[i] ) ; }
-		bool hasHoles() { return holes.size() > 0;}
+		bool hasHoles() { return !holes.empty();}
 		unsigned nHoles() { return holes.size();}
 		size_t ncoords();
 };
@@ -89,6 +91,7 @@ class SpatGeom {
 		//double length_plane();
 		//double length_lonlat(double a, double f);
 		unsigned size() { return parts.size(); };
+		bool empty() { return parts.empty(); };
 		void remove_duplicate_nodes(int digits);
 		size_t ncoords();
 		std::vector<std::vector<double>> coordinates();
@@ -166,6 +169,8 @@ class SpatVector {
 		std::vector<std::string> getGeometryWKT();
 		void computeExtent();
 
+		size_t nparts(bool holes);
+
 		size_t ncoords();
 		std::vector<std::vector<double>> coordinates();
 
@@ -194,11 +199,12 @@ class SpatVector {
 
 
 		std::vector<double> distance(bool sequential, std::string unit);
-		std::vector<double> linedistLonLat(SpatVector pts);
+		std::vector<double> linedistLonLat(SpatVector x);
 
 		std::vector<std::vector<size_t>> knearest(size_t k);
 
 		size_t size();
+		bool empty();
 		SpatVector as_lines();
 		SpatVector as_points(bool multi, bool skiplast=false);
 		SpatVector remove_holes();
@@ -310,8 +316,8 @@ class SpatVector {
 		SpatVector boundary();
 		SpatVector line_merge();
 		SpatVector simplify(double tolerance, bool preserveTopology);
-		SpatVector shared_paths();
-		SpatVector shared_paths(SpatVector x);
+		SpatVector shared_paths(bool index);
+		SpatVector shared_paths(SpatVector x, bool index);
 		SpatVector snap(double tolerance);
 		SpatVector snapto(SpatVector y, double tolerance);
 		SpatVector thin(double threshold);
@@ -321,7 +327,8 @@ class SpatVector {
 		SpatVector aggregate(bool dissolve);
 		SpatVector aggregate(std::string field, bool dissolve);
 
-        SpatVector buffer(std::vector<double> d, unsigned quadsegs);
+		SpatVector buffer(std::vector<double> d, unsigned quadsegs, std::string capstyle, std::string joinstyle, double mitrelimit, bool singlesided);
+
 		SpatVector point_buffer(std::vector<double>	 d, unsigned quadsegs, bool no_multipolygons);
 
 		SpatVector centroid(bool check_lonlat);
@@ -329,7 +336,7 @@ class SpatVector {
 
 		SpatVector crop(SpatExtent e);
 		SpatVector crop(SpatVector e);
-		SpatVector voronoi(SpatVector e, double tolerance, int onlyEdges);		
+		SpatVector voronoi(SpatVector bnd, double tolerance, int onlyEdges);		
 		SpatVector delaunay(double tolerance, int onlyEdges);		
 		SpatVector hull(std::string htype, std::string by="");
 		SpatVector intersect(SpatVector v, bool values);
@@ -379,6 +386,7 @@ class SpatVector {
 		SpatVector densify(double interval, bool adjust, bool ignorelonlat);
 		SpatVector round(int digits);
 		std::vector<unsigned> nullGeoms();
+				
 };
 
 
@@ -404,6 +412,7 @@ class SpatVectorCollection {
 		std::string getError() { return msg.getError();}
 
 		size_t size() { return v.size(); }
+		bool empty() { return v.empty(); }
 		void reserve(size_t n) { v.reserve(n); names.reserve(n); }
 		void resize(size_t n) { v.resize(n); names.resize(n); }
 		void push_back(SpatVector x) {
